Attribute VB_Name = "bas_Draw_Snowflake_s4p"
Option Compare Database
Option Explicit
'*************** Code Start ***************************************************
' Purpose  : draw a Snowflake on an Access report
'             specify center coordinate and radius
'             optionally set snowflake and background colors
'           USES Circle and Line
' Author   : crystal (strive4peace)
' Code List: www.msaccessgurus.com/code.htm
' This code: https://msaccessgurus.com/VBA/Draw_Snowflakes.htm
' LICENSE  :
'   You may freely use and share this code, but not sell it.
'   Keep attribution. Mark your changes. Use at your own risk.
'------------------------------------------------------------------------------
'           Global variables
'------------------------------------------------------------------------------
Public Const PI As Double = 3.14159

Public Const gColorCyan As Long = 16769385 'RGB(105, 225, 255)

'------------------------------------------------------------------------------
'           Draw_Snowflake_s4p
' send center coordinate and size
'------------------------------------------------------------------------------
Public Sub Draw_Snowflake_s4p(poReport As Report _
      , pXCenter As Single _
      , pYCenter As Single _
      , ByVal pRadius As Single _
   , Optional pnColor1 As Long = gColorCyan _
   , Optional pnColor2 As Long = 0 _
   , Optional psgAngleStart As Single = 0 _
   )
'221216 s4p
'Draw a Snowflake
'   measurements in twips

   On Error GoTo Proc_Err
   'PARAMETERs
   '  poReport is the Report object
   '  pXCenter is x-coordinate of snowflake center
   '  pYCenter is y-coordinate of snowflake center
   '  pRadius is snowflake radius
   
   '(Optional)
   '  pnColor1 = snowflake color
   '     Default is cyan
   '  pnColor2 = background color
   '     negative number is NO Background
   '     default is black circle background
      
   'X and Y are for Line coordinates
   'sgAngle is to calculate X and Y

   Dim X As Single, Y As Single _
      , x1 As Single, y1 As Single _
      , x2 As Single, y2 As Single _
      , sgAngle As Single _
      , sgAngleLeft As Single _
      , sgAngleRight As Single _
      , sgRadius1 As Single _
      , sgRadius2 As Single _
      , sgLength1 As Single _
      , sgLength2 As Single _
      , i As Integer

   'adjust radius to account for draw width
   pRadius = pRadius * 0.93

   '----------------------------- customize as desired
   sgRadius1 = pRadius / 3
   sgRadius2 = 2 * pRadius / 3
   sgLength1 = pRadius / 3
   sgLength2 = pRadius / 3
   sgAngleLeft = PI / 3
   sgAngleRight = -PI / 3
   '-----------------------------

    With poReport
      
      .ScaleMode = 1 'twips
      .DrawWidth = pRadius / 50 'relative based on size

      .FillStyle = 0 'Opaque

      If pnColor2 >= 0 Then
         'draw circle background
         .FillColor = pnColor2
         poReport.Circle (pXCenter, pYCenter) _
               , pRadius _
               , pnColor2
      End If
      
      'draw needles
      sgAngle = psgAngleStart
      '6 sides
      For i = 0 To 5
         X = pXCenter + Cos(sgAngle) * pRadius
         Y = pYCenter + Sin(sgAngle) * pRadius
         
         'big needle
         .DrawWidth = pRadius / 50
         poReport.Line (pXCenter, pYCenter)-(X, Y) _
            , pnColor1
            
         'inner little needles. x1, y1 same for both lines
         x1 = pXCenter + Cos(sgAngle) * sgRadius1
         y1 = pYCenter + Sin(sgAngle) * sgRadius1
         'left needle
         x2 = x1 + Cos(sgAngle + sgAngleLeft) * sgLength1
         y2 = y1 + Sin(sgAngle + sgAngleLeft) * sgLength1
         .DrawWidth = pRadius / 150
         poReport.Line (x1, y1)-(x2, y2), pnColor1
         'right needle
         x2 = x1 + Cos(sgAngle + sgAngleRight) * sgLength1
         y2 = y1 + Sin(sgAngle + sgAngleRight) * sgLength1
         poReport.Line (x1, y1)-(x2, y2), pnColor1
            
          'outer needles
         x1 = pXCenter + Cos(sgAngle) * sgRadius2
         y1 = pYCenter + Sin(sgAngle) * sgRadius2
         
         x2 = x1 + Cos(sgAngle + sgAngleLeft) * sgLength2
         y2 = y1 + Sin(sgAngle + sgAngleLeft) * sgLength2
         .DrawWidth = pRadius / 100
         poReport.Line (x1, y1)-(x2, y2), pnColor1
         x2 = x1 + Cos(sgAngle + sgAngleRight) * sgLength2
         y2 = y1 + Sin(sgAngle + sgAngleRight) * sgLength2
         poReport.Line (x1, y1)-(x2, y2), pnColor1
            
         'next angle
         sgAngle = sgAngle - 2 * PI / 6
      Next i

      
   End With
      
Proc_Exit:
   On Error GoTo 0
   Exit Sub
  
Proc_Err:
   MsgBox Err.Description _
       , , "ERROR " & Err.Number _
        & "   Draw_Snowflake_s4p"

   Resume Proc_Exit
   Resume
   
End Sub

'*************** Code End *****************************************************

